import { Component, h, Prop, Event } from '@stencil/core';
import { ControlInfoValue } from '../../utils/utils';
export class K2btToggleBar {
    constructor() {
        this.SmallChip = 'SmallChip';
        this.MediumChip = 'MediumChip';
        this.Rectangle = 'Rectangle';
        this.value = null;
        this.values = [];
        this.includeemptyitem = true;
        this.emptyitemtext = '(none)';
        this.noresultsfoundtext = 'No results found';
        this.enableadditem = true;
        this.additemcaption = 'New record';
        this.enabled = true;
        this.readonlyclass = '';
        this.togglestyle = this.MediumChip;
        this.selectedElement = null;
    }
    onImageError(e) {
        e.target.classList.add('K2BT_ToggleBarIconInvisible');
    }
    setValue(value) {
        this.value = value;
        this.inputEvent.emit();
        this.changeEvent.emit(value);
    }
    getToggleContent(values, containsDetails, containsIcons, containsTrailingText) {
        if (values.length == 0) {
            return h("div", { class: "K2BT_ToggleBar_NoItems" }, this.noresultsfoundtext);
        }
        else {
            const atomic = values.filter(v => { var _a, _b; return ((_b = (_a = v.items) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) == 0; });
            const currentLevelCategories = values.filter(v => { var _a; return ((_a = v.items) === null || _a === void 0 ? void 0 : _a.length) > 0; });
            var containerClasses = 'K2BT_ToggleBar_Container';
            if (this.togglestyle == this.SmallChip) {
                containerClasses += ' K2BT_ToggleBarSmallChip';
            }
            else if (this.togglestyle == this.Rectangle) {
                containerClasses += ' K2BT_ToggleBarRectangle';
            }
            return (h("div", { class: containerClasses },
                atomic.length > 0 ? (h("div", { class: "K2BT_ToggleBarItemContainer" }, atomic.map(item => (h("div", { class: this.value.trim() == item.value.trim() ? 'K2BT_ToggleBarItem K2BT_ToggleBarItemSelected' : 'K2BT_ToggleBarItem', ref: c => {
                        if (this.value.trim() == item.value.trim())
                            this.selectedElement = c;
                    }, onClick: () => this.setValue(item.value.trim()) },
                    containsIcons ? (h("div", { class: "K2BT_ToggleBarIconContainer" },
                        h("img", { class: "K2BT_ToggleBarIcon", src: item.imageUrl, onError: e => this.onImageError(e) }))) : (''),
                    h("div", { class: "K2BT_ToggleBarItem_TextSection" },
                        h("div", { class: "K2BT_ToggleBarItem_TextContainer" },
                            h("span", { class: "K2BT_ToggleBarDescription" }, item.description),
                            containsTrailingText ? h("span", { class: "K2BT_ToggleBarTrailingText" }, item.trailingText) : ''),
                        h("div", { class: "K2BT_ToggleBarItem_TextContainer" }, containsDetails ? h("span", { class: "K2BT_ToggleBarSubtitle" }, item.detail) : ''))))))) : (''),
                currentLevelCategories.map(c => {
                    return (h("div", { class: "K2BT_ToggleBar_CategoryContainer" },
                        h("div", { class: "K2BT_ToggleBarCategoryName" }, c.description),
                        this.getToggleContent(c.items, containsDetails, containsIcons, containsTrailingText)));
                })));
        }
    }
    getHeaderContent() {
        if (this.value == null)
            return this.emptyitemtext;
        else
            var vals = ControlInfoValue.getAtomicValues_impl(this.getValues()).filter(value => value.value.trim() == this.value.trim());
        if (vals.length > 0)
            return vals[0].description;
        else
            return this.emptyitemtext;
    }
    render() {
        var containsDetails = ControlInfoValue.containsDetails(ControlInfoValue.getAtomicValues_impl(this.values));
        var containsIcons = ControlInfoValue.containsIcons(ControlInfoValue.getAtomicValues_impl(this.values));
        var containsTrailingText = ControlInfoValue.containsTrailingText(ControlInfoValue.getAtomicValues_impl(this.values));
        var valueList = this.getValues();
        if (this.enabled) {
            return this.getToggleContent(valueList, containsDetails, containsIcons, containsTrailingText);
        }
        else {
            return (h("p", { class: "form-control-static" },
                h("span", { class: this.readonlyclass }, this.getHeaderContent())));
        }
    }
    getValues() {
        var valueList = [...this.values];
        if (this.includeemptyitem) {
            valueList.unshift({
                value: '',
                description: this.emptyitemtext,
                imageUrl: '',
                detail: '',
                trailingText: '',
                items: [],
            });
        }
        return valueList;
    }
    componentDidLoad() {
        var currentValues = ControlInfoValue.getAtomicValues_impl(this.values);
        if (!this.includeemptyitem && currentValues.filter(v => v.value.trim() == this.value.trim()).length == 0) {
            if (currentValues.length > 0) {
                this.setValue(currentValues[0].value.trim());
            }
        }
    }
    static get is() { return "k2bt-toggle-bar"; }
    static get originalStyleUrls() { return {
        "$": ["k2bt-toggle-bar.css"]
    }; }
    static get styleUrls() { return {
        "$": ["k2bt-toggle-bar.css"]
    }; }
    static get properties() { return {
        "value": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "value",
            "reflect": false,
            "defaultValue": "null"
        },
        "values": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "Array<ControlInfoValue>",
                "resolved": "ControlInfoValue[]",
                "references": {
                    "Array": {
                        "location": "global"
                    },
                    "ControlInfoValue": {
                        "location": "import",
                        "path": "../../utils/utils"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "defaultValue": "[]"
        },
        "includeemptyitem": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "includeemptyitem",
            "reflect": false,
            "defaultValue": "true"
        },
        "emptyitemtext": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "emptyitemtext",
            "reflect": false,
            "defaultValue": "'(none)'"
        },
        "noresultsfoundtext": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "noresultsfoundtext",
            "reflect": false,
            "defaultValue": "'No results found'"
        },
        "enableadditem": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "enableadditem",
            "reflect": false,
            "defaultValue": "true"
        },
        "additemcaption": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "additemcaption",
            "reflect": false,
            "defaultValue": "'New record'"
        },
        "enabled": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "enabled",
            "reflect": false,
            "defaultValue": "true"
        },
        "readonlyclass": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "readonlyclass",
            "reflect": false,
            "defaultValue": "''"
        },
        "togglestyle": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "togglestyle",
            "reflect": false,
            "defaultValue": "this.MediumChip"
        }
    }; }
    static get events() { return [{
            "method": "inputEvent",
            "name": "input",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": ""
            },
            "complexType": {
                "original": "object",
                "resolved": "object",
                "references": {}
            }
        }, {
            "method": "changeEvent",
            "name": "change",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": ""
            },
            "complexType": {
                "original": "object",
                "resolved": "object",
                "references": {}
            }
        }, {
            "method": "newRecordClickedEvent",
            "name": "newRecordClicked",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": ""
            },
            "complexType": {
                "original": "object",
                "resolved": "object",
                "references": {}
            }
        }]; }
}
